import click
from tqdm import tqdm
from mega import Mega
from mega.errors import RequestError

from .parser import Parser
from .user import User

mega = Mega()


@click.command()
@click.option(
    "--file",
    "-f",
    default="mega.txt",
    help="""
              Location of file which contains users and passwords of Mega. Default: mega.txt\n
              File should be formatted in form of [email][spaces/tabs][password].\n
              File may contain multiple accounts separated in ne line.\n
              Supports comment and empty lines (# or //).\n 
              i.e:\n
              bla@secure.com   mypassword\b\n
              mega@secure.com  securedpassword""",
)
@click.option(
    "--skip-fails",
    "-s",
    default=False,
    is_flag=True,
    type=bool,
    help="Don't exit upon failed accounts (wrong credentials, blocked user etc')",
)
def main(file: str, skip_fails: bool) -> None:
    try:
        users = Parser.parse_file(file)
    except Exception as e:
        print(str(e))
        raise click.Abort()

    users_progress_bar = tqdm(users)

    for user in users_progress_bar:
        try:
            users_progress_bar.set_description(f"Processing user {user.email}")
            login_user(user)
        except RuntimeError:
            if not skip_fails:
                raise click.Abort()
    print(f"Done!")


def login_user(user: User) -> None:
    try:
        logged_user = mega.login(user.email, user.password)
        logged_user.get_user()  # some action to do on the account without any side effects
    except RequestError as e:
        if e.code == -9:
            print(f"{user.email}: wrong email or password")
        elif e.code == -16:
            print(f"{user.email} is blocked")
        else:
            print(f"{user.email}: unknown exception in Mega.")
        raise RuntimeError(e)



import re
import pathlib
from typing import List

from .user import User


class Parser:
    @staticmethod
    def parse_file(file: str) -> List[User]:
        Parser._validate_file(file)
        return [User(*line.split()) for line in Parser._get_clean_file_in_lines(file)]

    @staticmethod
    def _validate_file(file: str) -> None:
        if not pathlib.Path(file).is_file():
            raise FileNotFoundError(f"file {file} not found")

        if faulty_lines := [line for line in Parser._get_clean_file_in_lines(file) if line.count(" ") != 1]:
            lines = "\n".join(faulty_lines)
            raise ValueError(f"file not formatted correctly. see lines:\n{lines}")

    @staticmethod
    def _get_clean_file_in_lines(file: str) -> List[str]:
        return [
            re.sub(r"\s+", " ", line.strip())
            for line in pathlib.Path(file).read_text().splitlines()
            if not Parser._is_empty_or_comment(line)
        ]

    @staticmethod
    def _is_empty_or_comment(string: str) -> bool:
        return (not string) or string.isspace() or string.strip().startswith("#") or string.strip().startswith("//")
